@@ -20,8 +20,10 @@ use File::Basename qw'dirname';
 use File::Spec;
 use File::Copy;
 use Rex::Repositorio::Repository_Factory;
+use JSON::XS;
+use Data::Dumper;
 
-our $VERSION = "0.2.0";
+our $VERSION = "0.3.0";
 
 has config => ( is => 'ro' );
 has logger => ( is => 'ro' );
@@ -61,6 +63,28 @@ sub parse_cli_option {
     $self->tag( tag => $option{tag}, repo => $option{repo} );
   }
 
+  elsif ( exists $option{repo} && exists $option{"update-errata"} ) {
+    $self->update_errata( repo => $option{repo} );
+  }
+
+  elsif ( exists $option{errata}
+    && exists $option{package}
+    && exists $option{arch}
+    && exists $option{repo}
+    && exists $option{version} )
+  {
+    $self->print_errata(
+      package => $option{package},
+      arch    => $option{arch},
+      version => $option{version},
+      repo    => $option{repo},
+    );
+  }
+
+  elsif ( exists $option{server} && exists $option{repo} ) {
+    $self->server( repo => $option{repo} );
+  }
+
   elsif ( exists $option{list} ) {
     $self->list();
   }
@@ -83,6 +107,31 @@ sub parse_cli_option {
   }
 }
 
+sub server {
+  my $self   = shift;
+  my %option = validate(
+    @_,
+    {
+      repo => {
+        type => SCALAR
+      },
+    }
+  );
+
+  require Mojolicious::Commands;
+
+  # pass config to mojo app
+  $ENV{'REPO_CONFIG'}           = encode_json( $self->config );
+  $ENV{'REPO_NAME'}             = $option{repo};
+  $ENV{'MOJO_MAX_MESSAGE_SIZE'} = 1024 * 1024 * 1024 * 1024
+    ;    # set max_message_size astronomically high / TODO: make it configurable
+  my $server_type = $self->config->{Repository}->{ $option{repo} }->{type};
+  if ( $server_type eq "Apt" ) {
+    $server_type = "Yum";
+  }
+  Mojolicious::Commands->start_app("Rex::Repositorio::Server::$server_type");
+}
+
 sub add_file {
   my $self   = shift;
   my %option = validate(
@@ -150,6 +199,82 @@ sub list {
   $self->_print(@repos);
 }
 
+sub update_errata {
+  my $self = shift;
+  my %option = validate(
+    @_,
+    {
+      repo => {
+        type => SCALAR
+      },
+    }
+  ); 
+
+  my $repo   = $self->config->{Repository}->{ $option{repo} };
+  my $type   = $repo->{type};
+  my $repo_o = Rex::Repositorio::Repository_Factory->create(
+    type    => $type,
+    options => {
+      app  => $self,
+      repo => {
+        name => $option{repo},
+        %{$repo},
+      }
+    }
+  );
+
+  $repo_o->update_errata();
+}
+
+sub print_errata {
+  my $self   = shift;
+  my %option = validate(
+    @_,
+    {
+      repo => {
+        type => SCALAR
+      },
+      package => {
+        type => SCALAR
+      },
+      version => {
+        type => SCALAR
+      },
+      arch => {
+        type => SCALAR
+      },
+    }
+  );
+
+  my $repo   = $self->config->{Repository}->{ $option{repo} };
+  my $type   = $repo->{type};
+  my $repo_o = Rex::Repositorio::Repository_Factory->create(
+    type    => $type,
+    options => {
+      app  => $self,
+      repo => {
+        name => $option{repo},
+        %{$repo},
+      }
+    }
+  );
+
+  my $errata = $repo_o->get_errata(
+    arch    => $option{arch},
+    package => $option{package},
+    version => $option{version}
+  );
+
+  for my $pkg_version ( sort { $a cmp $b } keys %{$errata} ) {
+    print "Name       : $errata->{$pkg_version}->[0]->{advisory_name}\n";
+    print "Version    : $pkg_version\n";
+    print "Synopsis   : $errata->{$pkg_version}->[0]->{synopsis}\n";
+    print "References : $errata->{$pkg_version}->[0]->{references}\n";
+    print "Type       : $errata->{$pkg_version}->[0]->{type}\n";
+    print "\n";
+  }
+}
+
 sub init {
   my $self   = shift;
   my %option = validate(
@@ -274,6 +399,25 @@ sub tag {
   }
 }
 
+sub get_errata_dir {
+  my $self   = shift;
+  my %option = validate(
+    @_,
+    {
+      repo => {
+        type => SCALAR
+      },
+      tag => {
+        type => SCALAR
+      }
+    }
+  );
+
+  return File::Spec->catdir(
+    File::Spec->rel2abs( $self->config->{RepositoryRoot} ),
+    $option{tag}, $option{repo}, "errata" );
+}
+
 sub get_repo_dir {
   my $self   = shift;
   my %option = validate(
@@ -313,6 +457,12 @@ sub _help {
     "--add-file=file     add a file to a repository (needs --repo)",
     "--remove-file=file  remove a file from a repository (needs --repo)",
     "--list              list known repositories",
+    "--server            start a server for file delivery. (not available for all repository types)",
+    "--update-errata     updates the errata database for a repo (needs --repo)",
+    "--errata            query errata for a package (needs --repo, --package, --version, --arch)",
+    "  --package=pkg     for which package the errata should be queries",
+    "  --version=ver     for which version of a package the errata should be queries",
+    "  --arch=arch       for which architecture of a package the errata should be queries",
     "--help              display this help message",
   );
 
@@ -341,11 +491,11 @@ create consistant installations of your server.
 
 =item * Web Site: L<http://repositor.io/>
 
-=item * IRC: irc.freenode.net #repositorio
+=item * IRC: irc.freenode.net #rex (RexOps IRC Channel)
 
-=item * Bug Tracker: L<https://github.com/krimdomu/repositorio/issues>
+=item * Bug Tracker: L<https://github.com/RexOps/repositorio/issues>
 
-=item * Twitter: L<http://twitter.com/jfried83>
+=item * Twitter: L<http://twitter.com/RexOps>
 
 =back
 
@@ -371,6 +521,18 @@ create consistant installations of your server.
 
 =item --list              list known repositories
 
+=item --server            start a server for file delivery. (not available for all repository types)
+
+=item --update-errata     updates the errata database for a repo (needs --repo)",
+
+=item --errata            query errata for a package (needs --repo, --package, --version, --arch)",
+
+=item --package=pkg       for which package the errata should be queries",
+
+=item --version=ver       for which version of a package the errata should be queries",
+
+=item --arch=arch         for which architecture of a package the errata should be queries",
+
 =item --help              display this help message
 
 =back
